@@ -33,7 +33,6 @@
 #include "xf86.h"
 #include "xf86Priv.h"
 #include "xf86_OSlib.h"
-#include "xf86Resources.h"
 #include "xf86cmap.h"
 
 #include "xf86Bus.h"
@@ -86,7 +85,7 @@ xf86SbusProbe(void)
     char fbDevName[32];
     sbusDevicePtr psdp, *psdpp;
 
-    xf86SbusInfo = xalloc(sizeof(psdp));
+    xf86SbusInfo = malloc(sizeof(psdp));
     *xf86SbusInfo = NULL;
     for (i = 0; i < 32; i++) {
 	sprintf(fbDevName, "/dev/fb%d", i);
@@ -211,7 +210,7 @@ xf86SbusProbe(void)
 	    promPath = sparcPromNode2Pathname (&psdp->node);
 	    if (promPath) {
 		xf86ErrorF(" at %s", promPath);
-		xfree(promPath);
+		free(promPath);
 	    }
 	} else
 	    xf86Msg(X_PROBED, "SBUS: %s", psdp->descr);
@@ -320,7 +319,7 @@ xf86CheckSbusSlot(int fbNum)
     for (i = 0; i < xf86NumEntities; i++) {
 	p = xf86Entities[i];
 	/* Check if this SBUS slot is taken */
-	if (p->busType == BUS_SBUS && p->sbusBusId.fbNum == fbNum)
+	if (p->bus.type == BUS_SBUS && p->bus.id.sbus.fbNum == fbNum)
 	    return FALSE;
     }
 
@@ -345,22 +344,18 @@ xf86ClaimSbusSlot(sbusDevicePtr psdp, DriverPtr drvp,
         p = xf86Entities[num];
         p->driver = drvp;
         p->chipset = -1;
-        p->busType = BUS_SBUS;
+        p->bus.type = BUS_SBUS;
         xf86AddDevToEntity(num, dev);
-        p->sbusBusId.fbNum = psdp->fbNum;
+        p->bus.id.sbus.fbNum = psdp->fbNum;
         p->active = active;
         p->inUse = FALSE;
-        /* Here we initialize the access structure */
-        p->access = xnfcalloc(1,sizeof(EntityAccessRec));
-	p->access->fallback = &AccessNULL;
-        p->access->pAccess = &AccessNULL;
 	sbusSlotClaimed = TRUE;
 	return num;
     } else
 	return -1;
 }
 
-_X_EXPORT int
+int
 xf86MatchSbusInstances(const char *driverName, int sbusDevId, 
 		       GDevPtr *devList, int numDevs, DriverPtr drvp,
 		       int **foundEntities)
@@ -402,15 +397,10 @@ xf86MatchSbusInstances(const char *driverName, int sbusDevId,
      * allow the config file to override this.
      */
     if (allocatedInstances <= 0) {
-	xfree(instances);
+	free(instances);
 	return 0;
     }
 
-    if (xf86DoProbe) {
-	xfree(instances);
-	return numFound;
-    }
-
     if (sparcPromInit() >= 0)
 	useProm = 1;
 
@@ -429,15 +419,13 @@ xf86MatchSbusInstances(const char *driverName, int sbusDevId,
 		pGDev->chipID = pGDev->chipRev = -1;
 	    }
 	}
-	xfree(instances);
+	free(instances);
 	if (useProm)
 	    sparcPromClose();
 	return actualcards;
     }
 
-#ifdef DEBUG
-    ErrorF("%s instances found: %d\n", driverName, allocatedInstances);
-#endif
+    DebugF("%s instances found: %d\n", driverName, allocatedInstances);
 
     for (i = 0; i < allocatedInstances; i++) {
 	char *promPath = NULL;
@@ -490,13 +478,10 @@ xf86MatchSbusInstances(const char *driverName, int sbusDevId,
 	    instances[i].claimed = TRUE;
 	    instances[i].dev = dev;
 	}
-	if (promPath)
-	    xfree(promPath);
+	free(promPath);
     }
 
-#ifdef DEBUG
-    ErrorF("%s instances found: %d\n", driverName, numClaimedInstances);
-#endif
+    DebugF("%s instances found: %d\n", driverName, numClaimedInstances);
 
     /*
      * Of the claimed instances, check that another driver hasn't already
@@ -510,11 +495,9 @@ xf86MatchSbusInstances(const char *driverName, int sbusDevId,
 	if (!xf86CheckSbusSlot(psdp->fbNum))
 	    continue;
 
-#ifdef DEBUG
-	ErrorF("%s: card at fb%d %08x is claimed by a Device section\n",
+	DebugF("%s: card at fb%d %08x is claimed by a Device section\n",
 	       driverName, psdp->fbNum, psdp->node.node);
-#endif
-	
+
 	/* Allocate an entry in the lists to be returned */
 	numFound++;
 	retEntities = xnfrealloc(retEntities, numFound * sizeof(int));
@@ -522,7 +505,7 @@ xf86MatchSbusInstances(const char *driverName, int sbusDevId,
 	    = xf86ClaimSbusSlot(psdp, drvp, instances[i].dev,instances[i].dev->active ?
 				TRUE : FALSE);
     }
-    xfree(instances);
+    free(instances);
     if (numFound > 0) {
 	*foundEntities = retEntities;
     }
@@ -536,38 +519,38 @@ xf86MatchSbusInstances(const char *driverName, int sbusDevId,
 /*
  * xf86GetSbusInfoForEntity() -- Get the sbusDevicePtr of entity.
  */
-_X_EXPORT sbusDevicePtr
+sbusDevicePtr
 xf86GetSbusInfoForEntity(int entityIndex)
 {
     sbusDevicePtr *psdpp;
     EntityPtr p = xf86Entities[entityIndex];
 
     if (entityIndex >= xf86NumEntities
-	|| p->busType != BUS_SBUS) return NULL;
+	|| p->bus.type != BUS_SBUS) return NULL;
 
     for (psdpp = xf86SbusInfo; *psdpp != NULL; psdpp++) {
-	if (p->sbusBusId.fbNum == (*psdpp)->fbNum)
-	    return (*psdpp);
+	if (p->bus.id.sbus.fbNum == (*psdpp)->fbNum)
+	    return *psdpp;
     }
     return NULL;
 }
 
-_X_EXPORT int
+int
 xf86GetEntityForSbusInfo(sbusDevicePtr psdp)
 {
     int i;
 
     for (i = 0; i < xf86NumEntities; i++) {
 	EntityPtr p = xf86Entities[i];
-	if (p->busType != BUS_SBUS) continue;
+	if (p->bus.type != BUS_SBUS) continue;
 
-	if (p->sbusBusId.fbNum == psdp->fbNum)
+	if (p->bus.id.sbus.fbNum == psdp->fbNum)
 	    return i;
     }
     return -1;
 }
 
-_X_EXPORT void
+void
 xf86SbusUseBuiltinMode(ScrnInfoPtr pScrn, sbusDevicePtr psdp)
 {
     DisplayModePtr mode;
@@ -602,8 +585,8 @@ xf86SbusUseBuiltinMode(ScrnInfoPtr pScrn, sbusDevicePtr psdp)
     pScrn->virtualY = psdp->height;
 }
 
-static int sbusPaletteIndex = -1;
-static unsigned long sbusPaletteGeneration = 0;
+static sbusPaletteKeyIndex;
+static DevPrivateKey sbusPaletteKey = &sbusPaletteKeyIndex;
 typedef struct _sbusCmap {
     sbusDevicePtr psdp;
     CloseScreenProcPtr CloseScreen;
@@ -613,7 +596,8 @@ typedef struct _sbusCmap {
     unsigned char origBlue[16];
 } sbusCmapRec, *sbusCmapPtr;
 
-#define SBUSCMAPPTR(pScreen) ((sbusCmapPtr)((pScreen)->devPrivates[sbusPaletteIndex].ptr))
+#define SBUSCMAPPTR(pScreen) ((sbusCmapPtr) \
+    dixLookupPrivate(&(pScreen)->devPrivates, sbusPaletteKey))
 
 static void
 xf86SbusCmapLoadPalette(ScrnInfoPtr pScrn, int numColors, int *indices,
@@ -622,7 +606,7 @@ xf86SbusCmapLoadPalette(ScrnInfoPtr pScrn, int numColors, int *indices,
     int i, index;
     sbusCmapPtr cmap;
     struct fbcmap fbcmap;
-    unsigned char *data = ALLOCATE_LOCAL(numColors*3);
+    unsigned char *data = malloc(numColors*3);
                              
     cmap = SBUSCMAPPTR(pScrn->pScreen);
     if (!cmap) return;
@@ -643,7 +627,7 @@ xf86SbusCmapLoadPalette(ScrnInfoPtr pScrn, int numColors, int *indices,
 	fbcmap.blue[fbcmap.count++] = colors[index].blue;
     }
     ioctl (cmap->psdp->fd, FBIOPUTCMAP, &fbcmap);
-    DEALLOCATE_LOCAL(data);
+    free(data);
 }
 
 static Bool
@@ -662,24 +646,19 @@ xf86SbusCmapCloseScreen(int i, ScreenPtr pScreen)
 	ioctl (cmap->psdp->fd, FBIOPUTCMAP, &fbcmap);
     }
     pScreen->CloseScreen = cmap->CloseScreen;
-    xfree (cmap);
+    free(cmap);
     return (*pScreen->CloseScreen) (i, pScreen);
 }    
 
-_X_EXPORT Bool
+Bool
 xf86SbusHandleColormaps(ScreenPtr pScreen, sbusDevicePtr psdp)
 {
     sbusCmapPtr cmap;
     struct fbcmap fbcmap;
     unsigned char data[2];
 
-    if(sbusPaletteGeneration != serverGeneration) {
-	if((sbusPaletteIndex = AllocateScreenPrivateIndex()) < 0)
-	    return FALSE;
-	sbusPaletteGeneration = serverGeneration;
-    }
     cmap = xnfcalloc(1, sizeof(sbusCmapRec));
-    pScreen->devPrivates[sbusPaletteIndex].ptr = cmap;
+    dixSetPrivate(&pScreen->devPrivates, sbusPaletteKey, cmap);
     cmap->psdp = psdp;
     fbcmap.index = 0;
     fbcmap.count = 16;
@@ -706,3 +685,30 @@ xf86SbusHandleColormaps(ScreenPtr pScreen, sbusDevicePtr psdp)
     return xf86HandleColormaps(pScreen, 256, 8,
 			       xf86SbusCmapLoadPalette, NULL, 0);
 }
+
+Bool
+xf86SbusConfigure(void *busData, sbusDevicePtr sBus)
+{
+    if (sBus && sBus->fbNum == ((sbusDevicePtr) busData)->fbNum)
+        return 0;
+    return 1;
+}
+
+void
+xf86SbusConfigureNewDev(void *busData, sbusDevicePtr sBus, GDevRec *GDev)
+{
+    char *promPath = NULL;
+
+    sBus = (sbusDevicePtr) busData;
+    GDev->identifier = sBus->descr;
+    if (sparcPromInit() >= 0) {
+        promPath = sparcPromNode2Pathname(&sBus->node);
+        sparcPromClose();
+    }
+    if (promPath) {
+        XNFasprintf(&GDev->busID, "SBUS:%s", promPath);
+        free(promPath);
+    } else {
+        XNFasprintf(&GDev->busID, "SBUS:fb%d", sBus->fbNum);
+    }
+}
