@@ -92,13 +92,13 @@ shadowUpdatePlanar4 (ScreenPtr	    pScreen,
 {
     RegionPtr	damage = shadowDamage (pBuf);
     PixmapPtr	pShadow = pBuf->pPixmap;
-    int		nbox = REGION_NUM_RECTS (damage);
-    BoxPtr	pbox = REGION_RECTS (damage);
+    int		nbox = RegionNumRects (damage);
+    BoxPtr	pbox = RegionRects (damage);
     CARD32	*shaBase, *shaLine, *sha;
     FbStride	shaStride;
     int		scrBase, scrLine, scr;
     int		shaBpp;
-    int		shaXoff, shaYoff;   /* XXX assumed to be zero */
+    _X_UNUSED int	shaXoff, shaYoff;
     int		x, y, w, h, width;
     int         i;
     CARD32	*winBase = NULL, *win;
